<?php
/**
 * @author jurgenhaas
 */

namespace GMJH\SQRL\Drupal7;

use GMJH\SQRL\ClientException;

/**
 * An override class for Drupal 7 SQRL account
 *
 * @author jurgenhaas
 *
 * @link
 */
class Account extends \GMJH\SQRL\Account {

  /**
   * @param Account $account
   * @return bool
   */
  protected function compare($account) {
    return ($this->get_uid() == $account->get_uid());
  }

  public function enabled() {
    return (bool) $this->account->status;
  }

  public function logged_in() {
    $sessions = db_select('sessions', 's')
      ->condition('s.uid', $this->account->uid)
      ->countQuery()
      ->execute()
      ->fetchField();
    return !empty($sessions);
  }

  public function get_uid() {
    return $this->account->uid;
  }

  public function multiple_accounts_allowed() {
    if (variable_get('sqrl_account_allow_multiple_per_id', SQRL_DEFAULT_ACCOUNT_ALLOW_MULTIPLE_PER_ID)) {
      return !$this->account->sqrl['restrict_multiple_accounts_per_id'];
    }
    return FALSE;
  }

  #region Command ==============================================================

  /**
   * @param Client $client
   * @param bool $additional
   * @return bool
   * @throws ClientException
   */
  public function command_setkey($client, $additional = FALSE) {
    if (!$this->account) {
      return FALSE;
    }
    $exists = db_select('sqrl_user', 'u')
      ->fields('u')
      ->condition('u.uid', $this->account->uid)
      ->condition('u.idk', $client->hash_pubkey($client->get_client_var('idk')))
      ->execute()
      ->fetchAll();
    if (!empty($exists)) {
      // Nothing changed and we have nothing to do.
      return TRUE;
    }
    if (!$additional) {
      // Remove the current key, we are getting a new one.
      db_delete('sqrl_user')
        ->condition('uid', $this->account->uid)
        ->condition('idk', $client->hash_pubkey($client->get_client_var('pidk')))
        ->execute();
    }

    // Save the new identity link.
    $values = array(
      'idk' => $client->hash_pubkey($client->get_client_var('idk')),
      'suk' => $client->get_client_var('suk'),
      'vuk' => $client->get_client_var('vuk'),
      'uid'  => $this->account->uid,
      'status' => TRUE,
    );
    if (drupal_write_record('sqrl_user', $values)) {
      $client->set_response('suk', $client->get_client_var('suk'));
      $client->set_response('vuk', $client->get_client_var('vuk'));
      return TRUE;
    }
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_setlock($client) {
    // TODO: How is this different from "setkey"?
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_disable($client) {
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_enable($client) {
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_delete($client) {
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_login($client) {
    if (($client->get_tif() & $client::FLAG_PIDK_MATCH) == $client::FLAG_PIDK_MATCH) {
      // The login request got authenticated with the PIDK. We do respond with
      // the SUK and VUK from the currently known ID and expect the client then
      // to issue a setkey request with the new SUK and VUK matching the new IDK.
      // TODO: When do we store the SUK and VUK with the account?
      $client->set_response('suk', $this->account->suk);
      $client->set_response('vuk', $this->account->vuk);
    }
    if (($client->get_tif() & $client::FLAG_ACCOUNT_DISABLED) != $client::FLAG_ACCOUNT_DISABLED) {
      $existing_accounts = $client->find_all_user_accounts($client->get_client_var('idk'));
      if ($client->get_sqrl()->get_operation() == 'register' && $this->multiple_accounts_allowed()) {
        // There already exists an account for the user but the registration
        // form was used instead of the login form. This may well mean that the
        // user deliberately wants to create another account and instead of
        // logging them in, ask them first.
        $client->get_sqrl()->set_operation_param('existing accounts', $existing_accounts);
        $client->get_sqrl()->ask_to_create_new_account($client);
        SQRL::get_message()->log(SQRL_LOG_LEVEL_DEBUG, 'Registration request with the ID of !name, do they want to create another account?', array('!name' => $this->account->name));
        $client->set_message('ID already known, server will ask you to create another account');
      }
      else if (count($existing_accounts) > 1) {
        $client->get_sqrl()->ask_to_select_account($client, $existing_accounts);
      }
      else {
        SQRL::get_message()->log(SQRL_LOG_LEVEL_DEBUG, 'Authentication for !name', array('!name' => $this->account->name));
        $client->set_message('Successfully logged in');
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_logme($client) {
    return FALSE;
  }

  /**
   * @param Client $client
   * @return bool
   * @throws ClientException
   */
  public function command_logoff($client) {
    return FALSE;
  }

  #endregion
}
