<?php
/**
 * @author jurgenhaas
 */

namespace GMJH\SQRL\Drupal7;

/**
 *
 */
class SQRL extends \GMJH\SQRL\SQRL {

  #region Private ==============================================================

  private $uid;
  private $destination;

  private function cid($mode) {
    switch ($mode) {
      case 'auth':
        $prefix = 'SQRL:';
        break;

      case 'nut':
        $prefix = 'SQRL:NUT:PARAMS:';
        break;

      default:
        $prefix = '';
        break;

    }
    return $prefix . $this->get_nut();
  }


  public function get_uid() {
    return $this->uid;
  }

  #endregion

  #region Abstract implementation ==============================================

  protected function build_base_url() {
    global $base_url;
    $scheme = file_uri_scheme($base_url);
    $sqrl_base_url = $scheme ? substr($base_url, strlen($scheme) + 3) : $base_url;
    return $sqrl_base_url;
  }

  public function is_secure_connection_available() {
    return $GLOBALS['is_https'];
  }

  public function encrypt($data, $is_cookie) {
    include_once 'sqrl.aes.inc';
    return sqrl_aes_encrypt($data, TRUE,
      sqrl_aes_get($is_cookie, 'key'),
      sqrl_aes_get($is_cookie, 'cipher'),
      sqrl_aes_get($is_cookie, 'iv'),
      sqrl_aes_get($is_cookie, 'implementation')
    );
  }

  public function decrypt($data, $is_cookie) {
    include_once 'sqrl.aes.inc';
    return sqrl_aes_decrypt($data, TRUE,
      sqrl_aes_get($is_cookie, 'key'),
      sqrl_aes_get($is_cookie, 'cipher'),
      sqrl_aes_get($is_cookie, 'iv'),
      sqrl_aes_get($is_cookie, 'implementation')
    );
  }

  public function save($params) {
    cache_set($this->cid('nut'), $params, 'cache', $this->get_timeout());
  }

  public function load() {
    $cache = cache_get($this->cid('nut'), 'cache');
    return empty($cache->data) ? FALSE : $cache->data;
  }

  public function counter() {
    $counter = variable_get('sqrl_nut_counter', 0);
    //Reset on 32 bit rollover
    if ($counter > (2 ^ 32 - 1)) {
      $counter = 0;
    }
    variable_set('sqrl_nut_counter', $counter + 1);
    return $counter;
  }

  /**
   * @param Account $account
   */
  public function authenticate($account) {
    if (is_numeric($account)) {
      // Called from sqrl_create_submit().
      $this->uid = $account;
    }
    else {
      $this->uid = $account->get_uid();
      cache_set($this->cid('auth'), $this->uid, 'cache', $this->get_timeout());
    }
  }

  protected function authenticated() {
    $cache = cache_get($this->cid('auth'), 'cache');
    if (!empty($cache->data)) {
      $this->uid = $cache->data;
      cache_clear_all($this->cid('auth'), 'cache');
      $this->perform();
      return TRUE;
    }
    return FALSE;
  }

  public function get_authenticated_destination() {
    return $this->destination;
  }

  /**
   * @return bool
   */
  public function is_auto_create_account() {
    return ($this->get_operation() == 'register');
  }

  /**
   * Create a new user account and maybe also authenticate the request, if
   * no administrator approval is required.
   *
   * @param Client $client
   * @return bool
   *  Return FALSE so that Client::commands_execute() will not authenticate
   *  again.
   */
  public function create_new_account($client) {
    $userinfo = array(
      'name' => '',
      'pass' => user_password(32),
      'init' => '',
      'status' => (variable_get('user_register') == 1),
      'access' => $this->get_request_time(),
    );
    $pattern = variable_get('sqrl_accounts_name_pattern', 'SQRL/[rand]');
    if (strpos($pattern, '[rand]') === FALSE) {
      $pattern .= '[rand]';
    }
    while (empty($user)) {
      $name = str_replace('[rand]', drupal_random_key(variable_get('sqrl_accounts_name_random_length', 5)), $pattern);
      if (!user_load_by_name($name)) {
        $userinfo['name'] = $userinfo['init'] = $name;
        $userinfo['mail'] = _sqrl_dummy_mail($name);
        try {
          $user = user_save(drupal_anonymous_user(), $userinfo);
        }
        catch (\Exception $ex) {
          // Do nothing, just retry.
        }
      }
    }
    $account = new Account($user);
    $account->command_setkey($client);
    $this->add_message_to_browser('info', 'Successfully created user account', FALSE);
    SQRL::get_message()->msg('info', 'Successfully created user account');
    sqrl_log(WATCHDOG_NOTICE, 'Account created for !name', array('!name' => $user->name), l(t('View profile'), 'user/' . $user->uid));
    if (variable_get('user_register') > 1) {
      // New accounts require admin approval.
      $this->add_message_to_browser('info', 'Account created, please wait for an administrator for approval of your account', FALSE);
      SQRL::get_message()->msg('info', 'Account created, please wait for an administrator for approval of your account');
    }
    else {
      $this->authenticate($account);
    }
    return FALSE;
  }

  #endregion

  #region Overwrites ===========================================================

  public function use_secure_connection() {
    return $this->is_secure_connection_available() ?
      variable_get('sqrl_use_secure_connection', SQRL_DEFAULT_USE_SECURE_CONNECTION) :
      FALSE;
  }

  public function get_connection_port() {
    return $this->use_secure_connection() ?
      variable_get('sqrl_secure_connection_port', SQRL_DEFAULT_SECURE_CONNECTION_PORT) :
      variable_get('sqrl_insecure_connection_port', SQRL_DEFAULT_INSECURE_CONNECTION_PORT);
  }

  public function is_page_cached() {
    return (variable_get('cache', FALSE) && drupal_page_is_cacheable());
  }

  public function get_qr_size() {
    return variable_get('sqrl_qr_size', SQRL_DEFAULT_IMAGE_SIZE);
  }

  public function get_poll_interval($initial = FALSE) {
    return $initial ? variable_get('sqrl_poll_interval_initial', SQRL_DEFAULT_POLL_INTERVAL_INITIAL) : variable_get('sqrl_poll_interval', SQRL_DEFAULT_POLL_INTERVAL);
  }

  #endregion

  /**
   * Perform the authenticated operation when the browser next requests a page and
   * the authentication had already been confirmed.
   *
   * @return string
   */
  public function perform() {
    $op = $this->get_operation();
    $this->destination = '<front>';
    switch ($op) {
      case 'login':
      case 'register':
        // Log user in.
        // If the operation was to register a new user, then the new user account
        // got created by the authentication and hence we also just need to login
        // the user when performing that operation.
        $form_state['uid'] = $this->uid;
        user_login_submit(array(), $form_state);
        $this->destination = $form_state['redirect'];
        drupal_set_message('You have been successfully logged in with your SQRL identity.');
        sqrl_log(WATCHDOG_INFO, 'Login user !uid', array('!uid' => $this->uid,));
        sqrl_user_session_destroy_others($this->uid, TRUE);
        break;

      case 'link':
        // User linked their account to their SQRL identity.
        $this->destination = 'user/' . $this->uid;
        sqrl_log(WATCHDOG_INFO, 'Linked user !uid to SQRL', array('!uid' => $this->uid,));
        break;

      default:
        foreach (module_implements('sqrl_perform') as $module) {
          sqrl_log(WATCHDOG_INFO, 'Handing over request for user !uid to !module', array(
            '!uid' => $this->uid,
            '!module' => $module
          ));
          $result = module_invoke($module, 'sqrl_perform', $op, $this->uid);
          if (!empty($result)) {
            $this->destination = $result;
            break;
          }
        }

    }

    $this->destination = url($this->destination, array('absolute' => TRUE));
    return $this->destination;
  }

}
